function DisplayParts(disp_img, parts, part_scale, part_score)
    [M,N] = size(parts(:,:,1));
    num_parts = size(parts,3);
    num_scales = max(part_scale);
    num_cols = ceil(sqrt(num_scales));
    num_rows = ceil(num_scales / num_cols);
    
    part_label_img = zeros(M, N, num_scales);
    part_img = zeros(M, N, 3, num_scales);
    
    parts_idx = parts .* repmat(reshape(1:num_parts, [1,1,num_parts]), [M, N, 1]);
    
    % Compute the parts label image for each scale
    for i = 1:num_scales
        part_label_img(:,:,i) = sum(parts_idx(:,:,part_scale == i), 3);
        boundary = bwmorph(seg2bmap(part_label_img(:,:,i)), 'dilate', 1);
        boundary_dil = bwmorph(seg2bmap(part_label_img(:,:,i)), 'dilate', 2);
        cur_part_img = im2double(label2rgb(part_label_img(:,:,i)));
        background_pixel_idx = repmat(part_label_img(:,:,i), [1,1,3]) == 0;
        cur_part_img(background_pixel_idx) = disp_img(background_pixel_idx);
        cur_part_img(repmat(boundary_dil,[1,1,3])) = 0;
        cur_part_img(repmat(boundary,[1,1,3])) = 1;
        
        part_img(:,:,:,i) = cur_part_img;
    end
    
    % Title subplot
    subplot('Position',[0.4 0.95 0.01 0.01]);
    axis off;
    text(0,0, 'Detected parts', 'Fontsize', 30);
    
    % Display the parts at each scale
    plots = zeros(num_scales, 1);
    for i = 1:num_scales
%         subplot(num_rows, num_cols, i);
        row = floor((i-1) / num_cols);
        col = mod((i-1), num_cols);
        
        plots(i) = subplot('Position',[col*0.5 (num_rows-row-1)*0.5 0.5 0.5]);
        iptsetpref('ImshowBorder', 'tight')
        imshow(part_img(:,:,:,i));
    end

    scrsz = get(0,'ScreenSize');
    img_aspect_ratio = N/M;
    screen_aspect_ratio = scrsz(3) / scrsz(4);
    
    if (img_aspect_ratio < screen_aspect_ratio)
        set(gcf, 'Position',[1 1 img_aspect_ratio*scrsz(4) scrsz(4)]);
    else
        set(gcf, 'Position',[1 1 scrsz(3) scrsz(3)/img_aspect_ratio]);
    end
    
    if (~isempty(part_score))
        % Compute the part center for each part
        part_centers = zeros(num_parts, 2);
        for i = 1:num_parts
            stats = regionprops(parts(:,:,i), 'Centroid');
            part_centers(i, :) = stats(1).Centroid;
        end

        for i = 1:num_parts
            subplot(plots(part_scale(i)));
            
            text(part_centers(i, 1), part_centers(i,2), num2str(part_score(i),'%4.4g'), ...
                'Fontsize', 15, 'Color', 'k', 'BackgroundColor', [0.5,0.5,0.5]); 
        end
    end