function DrawPartClustersEllipses(img, parts, part_labels, selected_parts_indicator)

    num_parts = size(parts, 3);
    num_clusters = max(part_labels);
    
    if (nargin < 4 || isempty(selected_parts_indicator))
        selected_parts_indicator = true(num_parts, 1);
    end
    
    draw_other_parts = true;
    
    if (num_clusters <= 5)
        part_color_map = [[1,0,0];[0,1,0];[0,0,1];[0.5,0.5,0];[0,0.5,0.5]];
    else
        part_color_map = colormap(hsv(num_clusters));
    end
    
    clf;
    imshow(img);
    axis_handle = gca;
    
    hold on;
    
    % Draw all the background parts (ones not selected for the clusters)
    if (draw_other_parts)
        for i = 1:num_parts
            if (~selected_parts_indicator(i))
                part_ellipse = PartEllipse(parts(:,:,i));           
                DrawEllipse(part_ellipse, [1,1,1], 3, axis_handle, true, [0.2,0.2,0.2]);
            end
        end
    end
    
    % Draw all the part ellipses first
    for i = 1:num_parts
        if (selected_parts_indicator(i))
            part_ellipse = PartEllipse(parts(:,:,i));           
            DrawEllipse(part_ellipse, 'y', 5, axis_handle, true);
            
%             part_stats = regionprops(double(seg2bmap(parts(:,:,i))), 'PixelList');
%             
%             scatter(part_stats.PixelList(:,1), part_stats.PixelList(:,2), 70, 'w', 'filled');
%             scatter(part_stats.PixelList(:,1), part_stats.PixelList(:,2), 50, 'k', 'filled');
        end
    end
    
    % Draw the medial axes next
    for i = 1:num_parts
        if (selected_parts_indicator(i))
            axis_line = PartMedialAxisLinear(parts(:,:,i));

            line([axis_line.x1; axis_line.x2], [axis_line.y1; axis_line.y2], ...
                'Linewidth', 13, 'Color', 'k');

            line([axis_line.x1; axis_line.x2], [axis_line.y1; axis_line.y2], ...
                'Linewidth', 10, 'Color', part_color_map(part_labels(i), :) , 'LineStyle', '--');
        end
    end
