% PartAffinityCluster - Cluster parts based on their affinties
% with a method similar to the Felz. and Hut. method
% 
function labels = PartAffinityCluster(weights, k, parts, img, affinity, adjmat, image_data)

    display_on = false;
    [part1, part2] = find(tril(adjmat, -1));
    [sorted_weights, sorted_idx] = sort(weights(sub2ind(size(weights), part1, part2)), 'ascend');
    
    num_parts = size(weights, 1);
    labels = (1:num_parts)';
    max_weights = zeros(size(labels));
    component_size = ones(size(labels));
    
    num_edges = numel(part1);
    
    % Merge edges in decreasing order of affinity
    for i = 1:num_edges
        cur_part1 = part1(sorted_idx(i));
        cur_part2 = part2(sorted_idx(i));
        
        label1 = labels(cur_part1);
        label2 = labels(cur_part2);
        
        if (label1 == label2)
            continue;
        end
        
        cur_max_weight = min(max_weights(cur_part1) + k/component_size(cur_part1), ...
            max_weights(cur_part2) + k/component_size(cur_part2));
        cur_weight = weights(cur_part1, cur_part2);
        
        if (cur_weight <= cur_max_weight)
            
            elements1 = labels == label1;
            elements2 = labels == label2;
            
            new_label = min(label1, label2);
            all_elements = elements1 | elements2;

            elements1_idx = find(elements1);
            elements2_idx = find(elements2);
            elements_all_idx = find(all_elements);
              
%             closure1 = PartsClosure(parts, image_data, elements1_idx);
%             closure2 = PartsClosure(parts, image_data, elements2_idx);
%             closure_all = PartsClosure(parts, image_data, elements_all_idx);
%             closure_improvement = closure_all / max(closure1, closure2);
%             
%             if (closure_improvement < 0.9)
%                 continue;
%             end
            
            selected_edges1 = ismember(part1, elements1_idx) & ismember(part2, elements1_idx);
            selected_edges2 = ismember(part1, elements2_idx) & ismember(part2, elements2_idx);
            selected_edges_all = ismember(part1, elements_all_idx) & ismember(part2, elements_all_idx);
            
            min_affinity1 = min(affinity(sub2ind(size(affinity), part1(selected_edges1), part2(selected_edges1))));
            min_affinity2 = min(affinity(sub2ind(size(affinity), part1(selected_edges2), part2(selected_edges2))));
            min_affinity_all = min(affinity(sub2ind(size(affinity), part1(selected_edges_all), part2(selected_edges_all))));
            old_min_affinity = min(min_affinity1, min_affinity2);
            if (isempty(old_min_affinity) || isnan(old_min_affinity))
                old_min_affinity = 1;
            end
            new_min_affinity = min_affinity_all;
            
%             disp(['The minimal affinity in the resulting cluster - was (', num2str(old_min_affinity),'), is (', num2str(new_min_affinity),')']);
%             if (new_min_affinity < 0.5*old_min_affinity)
%                 disp(['Unsuccessful merge of parts (', num2str(cur_part1), ',', num2str(cur_part2),'), Current affinity: ', num2str(affinity(cur_part1, cur_part2))]);
% 
%                 continue;
%             end
            
            labels(all_elements) = new_label;
            max_weights(all_elements) = cur_weight;
            component_size(all_elements) = sum(all_elements);
            
            if (display_on)
                part_cluster_BW = sum(parts(:,:,all_elements), 3) > 0;
                part_cluster_img = img;
                part_cluster_img(repmat(part_cluster_BW,[1,1,3])) = 1;

                imshow(part_cluster_img);
                disp(['Merging parts (', num2str(cur_part1), ',', num2str(cur_part2),'), Current affinity: ', num2str(affinity(cur_part1, cur_part2))]);

%                 disp(['Current closure improvement: ', num2str(closure_improvement)]);
                
                pause;
            end
        end
    end
    
    unique_labels = unique(labels(:));
    new_labels = zeros(size(labels));
    num_labels = numel(unique_labels);

    for label = 1:num_labels
        idx = (labels == unique_labels(label));
        new_labels(idx) = label;
    end
    labels = new_labels;
    