function features = PartAffinityFeatures(part1_BW, part2_BW, part1_feature, part2_feature, image_data)

    load classifier_params_svm;
    
    [M,N] = size(part1_BW);
    
    joint_region_BW = part1_BW | part2_BW;
    intersection_region_BW = part1_BW & part2_BW;
    
    % Compute the intersection length
    dilated_part1 = part1_feature.dilated_part_small;
    dilated_part2 = part2_feature.dilated_part_small;
    
    boundary1 = part1_feature.boundary;
    boundary2 = part2_feature.boundary;
    boundary_joint = bwperim(joint_region_BW);
    
    boundary_length_1_in_2 = sum(dilated_part2(boundary1(:)));
    boundary_length_2_in_1 = sum(dilated_part1(boundary2(:)));
    
    boundary1_length = part1_feature.perim_length;
    boundary2_length = part2_feature.perim_length;
    joint_length = sum(boundary_joint(:));
    
    min_intersection = min([boundary_length_1_in_2, boundary_length_2_in_1]);
    max_intersection = max([boundary_length_1_in_2, boundary_length_2_in_1]);
    mean_intersection = mean([boundary_length_1_in_2, boundary_length_2_in_1]);

    rel_intersection = mean_intersection / min([boundary1_length, boundary2_length]);
    
    
    % Compute overlap
    area1 = part1_feature.area;
    area2 = part2_feature.area;
    overlap = sum(intersection_region_BW(:)) / min([area1, area2]);
    
    % Compute bearing
    
    % Fit ellipses to parts
    vec1_2 = [part2_feature.x0 - part1_feature.x0; part2_feature.y0 - part1_feature.y0];
    vec1_2 = vec1_2 / norm(vec1_2);
    vec2_1 = -vec1_2;
    
    bearing1 = acos(abs(cos(part1_feature.ellipse_angle)*vec1_2(1) + sin(part1_feature.ellipse_angle)*vec1_2(2)));
    bearing2 = acos(abs(cos(part2_feature.ellipse_angle)*vec2_1(1) + sin(part2_feature.ellipse_angle)*vec2_1(2)));
    
    % Compute the angle between the ellipse major axes
    relative_angle = acos(abs(cos(part1_feature.ellipse_angle)*cos(part2_feature.ellipse_angle) + ...
        sin(part1_feature.ellipse_angle)*sin(part2_feature.ellipse_angle)));
    
    if (isnan(bearing1) || isnan(bearing2) || ~isreal(relative_angle))
        disp('Shitty angle encountered');
    end
    
    % Compute the overlap between dilated parts
    % Dilate each part in proportion to its minor axis
    % Check if the other part is overlaping the dilated part
    % Dliated overlap should be large if two parallel parts are very close
    % to each other, side by side.
    overlap1 = sum(part1_BW(:) & part2_feature.dilated_part_small(:)) / min([area1, part2_feature.area_dil_small]);
    overlap2 = sum(part2_BW(:) & part1_feature.dilated_part_small(:)) / min([area2, part1_feature.area_dil_small]);
    dilated_overlap = max([overlap1, overlap2]);
    
    % Compute a larger dilated overlap for the purpose of determining
    % neighbourhood relations between parts
    overlap1 = sum(part1_BW(:) & part2_feature.dilated_part_big(:)) / min([area1, part2_feature.area_dil_big]);
    overlap2 = sum(part2_BW(:) & part1_feature.dilated_part_big(:)) / min([area2, part1_feature.area_dil_big]);
    dilated_overlap_large = min([overlap1, overlap2]);
    
    % Compute size difference
    relative_size = min(part1_feature.b, part2_feature.b) / max(part1_feature.b, part2_feature.b);
    
    % The distance between the major axes
    axis1 = [part1_feature.x0 + cos(part1_feature.ellipse_angle)*part1_feature.a; ...
        part1_feature.y0 + sin(part1_feature.ellipse_angle)*part1_feature.a; ...
        part1_feature.x0 - cos(part1_feature.ellipse_angle)*part1_feature.a; ...
        part1_feature.y0 - sin(part1_feature.ellipse_angle)*part1_feature.a];
    
    axis2 = [part2_feature.x0 + cos(part2_feature.ellipse_angle)*part2_feature.a; ...
        part2_feature.y0 + sin(part2_feature.ellipse_angle)*part2_feature.a; ...
        part2_feature.x0 - cos(part2_feature.ellipse_angle)*part2_feature.a; ...
        part2_feature.y0 - sin(part2_feature.ellipse_angle)*part2_feature.a];
    
    [axes_distance_closest, axes_distance_inter, is_projected_in, junc_point] = LineSegmentDistance(axis1, axis2);
%     axes_distance_closest = axes_distance_closest / max(part1_feature.b, part2_feature.b);
%     axes_distance_inter = axes_distance_inter / max(part1_feature.b, part2_feature.b);
    d = zeros(4,1);
    d(1) = norm(axis1(1:2) - junc_point);
    d(2) = norm(axis2(3:4) - junc_point);
    d(3) = norm(axis2(1:2) - junc_point);
    d(4) = norm(axis2(3:4) - junc_point);
    axis_to_inter_dist = min(d);
   
    % The distance between two closest axes endpoints
    d = zeros(4,1);
    d(1) = norm(axis1(1:2) - axis2(1:2));
    d(2) = norm(axis1(1:2) - axis2(3:4));
    d(3) = norm(axis1(3:4) - axis2(1:2));
    d(4) = norm(axis1(3:4) - axis2(3:4));
    min_axis_dist = min(d);
    
    % Middle to junction point distance
    mid_to_junc1 = norm((axis1(1:2) + axis1(3:4))/2 - junc_point) / ...
        part1_feature.a;
    mid_to_junc2 = norm((axis2(1:2) + axis2(3:4))/2 - junc_point) / ...
        part2_feature.a;
    
    ins_limit = 0.5;
    out_limit = 1.5;
    is_t_junction = (mid_to_junc1 < ins_limit & mid_to_junc2 > ins_limit) | ...
        (mid_to_junc2 < ins_limit & mid_to_junc1 > ins_limit);
    is_cross_junction = mid_to_junc1 < ins_limit & mid_to_junc2 < ins_limit;
    is_end_to_end_junction = (mid_to_junc1 > ins_limit & mid_to_junc1 < out_limit) & ...
        (mid_to_junc2 > ins_limit & mid_to_junc2 < out_limit);
    
    if (is_end_to_end_junction)
        junction_type = 1;
    elseif (is_t_junction)
        junction_type = 2;
    elseif (is_cross_junction)
        junction_type = 3;
    else
        junction_type = 4;
    end
    
    % Normalized axes lengths
    major_axis1_length = part1_feature.a;
    major_axis2_length = part2_feature.a;
    minor_axis1_length = part1_feature.b;
    minor_axis2_length = part2_feature.b;
    
    % Shared boundary
    dil_boundary1 = part1_feature.dil_boundary;
    dil_boundary2 = part2_feature.dil_boundary;
    boundary1_in_2 = boundary1 & dil_boundary2;
    boundary2_in_1 = boundary2 & dil_boundary1;
    boundary_inter_pixels = boundary1_in_2 | boundary2_in_1;
    boundary1_intersection = sum(boundary1_in_2(:)) / min(boundary1_length, boundary2_length);
    boundary2_intersection = sum(boundary2_in_1(:)) / min(boundary1_length, boundary2_length);
    boundary_intersection = max(boundary1_intersection, boundary2_intersection);
    boundary_intersection = min(boundary_intersection, 1);
    
    % Edge between parts
    [junction_point, gap_points] = PartPairIntersection(part1_feature, part2_feature);
    mean_edge_support_projected = mean(image_data.dilated_pb(sub2ind([M,N], gap_points(:,2), gap_points(:,1))));
    mean_edge_support_attached = mean(image_data.dilated_pb(boundary_inter_pixels(:)));
    
    % Appearance affinity
    if (nargin > 4)
        num_bins_color = 10;

        color_hist1 = RegionColorHistogram(part1_BW, image_data.img_h, image_data.img_s, image_data.img_v, num_bins_color);
        color_hist2 = RegionColorHistogram(part2_BW, image_data.img_h, image_data.img_s, image_data.img_v, num_bins_color);
        mean_r1 = mean(image_data.img_r(part1_BW));
        mean_g1 = mean(image_data.img_g(part1_BW));
        mean_b1 = mean(image_data.img_b(part1_BW));
        var_r1 = var(image_data.img_r(part1_BW));
        var_g1 = var(image_data.img_g(part1_BW));
        var_b1 = var(image_data.img_b(part1_BW));
        mean_r2 = mean(image_data.img_r(part2_BW));
        mean_g2 = mean(image_data.img_g(part2_BW));
        mean_b2 = mean(image_data.img_b(part2_BW));
        var_r2 = var(image_data.img_r(part2_BW));
        var_g2 = var(image_data.img_g(part2_BW));
        var_b2 = var(image_data.img_b(part2_BW));

        mean_cos1 = mean(image_data.cos_hs(part1_BW));
        mean_sin1 = mean(image_data.sin_hs(part1_BW));
        mean_v1 = mean(image_data.img_v(part1_BW));
        var_cos1 = var(image_data.cos_hs(part1_BW));
        var_sin1 = var(image_data.sin_hs(part1_BW));
        var_v1 = var(image_data.img_v(part1_BW));
        mean_cos2 = mean(image_data.cos_hs(part2_BW));
        mean_sin2 = mean(image_data.sin_hs(part2_BW));
        mean_v2 = mean(image_data.img_v(part2_BW));
        var_cos2 = var(image_data.cos_hs(part2_BW));
        var_sin2 = var(image_data.sin_hs(part2_BW));
        var_v2 = var(image_data.img_v(part2_BW));

        mean_color1 = [mean_r1, mean_g1, mean_b1];
        var_color1 = [var_r1, var_g1, var_b1];
        mean_color2 = [mean_r2, mean_g2, mean_b2];
        var_color2 = [var_r2, var_g2, var_b2];

        mean_color_hsv1 = [mean_cos1, mean_sin1, mean_v1];
        var_color_hsv1 = [var_cos1, var_sin1, var_v1];
        mean_color_hsv2 = [mean_cos2, mean_sin2, mean_v2];
        var_color_hsv2 = [var_cos2, var_sin2, var_v2];

        color_affinity_h = AffinityColorHistogram(color_hist1(1:num_bins_color), color_hist2(1:num_bins_color));
        color_affinity_s = AffinityColorHistogram(color_hist1(num_bins_color+(1:num_bins_color)), color_hist2(num_bins_color+(1:num_bins_color)));
        color_affinity_v = AffinityColorHistogram(color_hist1(2*num_bins_color+(1:num_bins_color)), color_hist2(2*num_bins_color+(1:num_bins_color)));
        dark1 = 1-mean_v1;
        dark2 = 1-mean_v2;
        is_bright = (1-dark1)*(1-dark2);
        similar_brightness = color_affinity_v;

        color_features = [abs(mean_color1 - mean_color2), ...
            abs(var_color1 - var_color2), ...
            var_color1, var_color2, ...
            abs(mean_color_hsv1 - mean_color_hsv2), ...
            abs(var_color_hsv1 - var_color_hsv2), ...
            var_color_hsv1, var_color_hsv2, ...
            is_bright*color_affinity_h, is_bright*color_affinity_s, (1-is_bright)*similar_brightness, 1];

        num_features = numel(color_features);
        idx = tril(ones(num_features, num_features)) > 0;
        quad_features = color_features' * color_features;
        color_features = quad_features(idx(:))';

        affinity_color = LogisticPredict(color_features, classifier_params.logistic_color);

        features = [overlap; rel_intersection; bearing1; bearing2; relative_angle; dilated_overlap; ...
            dilated_overlap_large; relative_size; affinity_color; axes_distance_closest; ...
            axes_distance_inter; axis_to_inter_dist; is_projected_in; boundary_intersection; ...
            min_axis_dist; major_axis1_length; major_axis2_length; minor_axis1_length; minor_axis2_length; ...
            mean_edge_support_projected; mean_edge_support_attached; junction_type];
    else    
        features = [overlap; rel_intersection; bearing1; bearing2; relative_angle; dilated_overlap; ...
            dilated_overlap_large; relative_size; axes_distance_closest; ...
            axes_distance_inter; axis_to_inter_dist; is_projected_in; boundary_intersection; ...
            min_axis_dist; major_axis1_length; major_axis2_length; minor_axis1_length; minor_axis2_length; ...
            mean_edge_support_projected; mean_edge_support_attached; junction_type];
    end