function [part_affinity, part_adjacency, part_pair_features] = PartAffinityOneImage(img, parts, part_classifier_params, image_data, compute_only_features)

    if (nargin < 5)
        compute_only_features = false;
    end
    
    num_parts = size(parts,3);
    part_affinity = zeros(num_parts, num_parts);
    part_adjacency = zeros(num_parts, num_parts);
    single_part_closure = zeros(num_parts, 1);
    num_features = 22;
    part_pair_features = zeros(num_parts, num_parts, num_features);
    
    for i = 1:num_parts
        single_part_features(i) = PartFeature(parts(:,:,i)>0);
%         single_part_closure(i) = PartsClosure(parts, image_data, i);
    end
    
    for i = 1:num_parts
        for j = 1:i-1
            if (sum(single_part_features(i).boundary(:) ~= single_part_features(j).boundary(:)) > 0)
                part_features = PartAffinityFeatures(parts(:,:,i)>0, parts(:,:,j)>0, single_part_features(i), single_part_features(j), image_data);
%                 closure = PartsClosure(parts, image_data, [i,j]);
% 
%                 closure_feature = closure;
                part_pair_features(i,j,:) = part_features;
                part_pair_features(j,i,:) = part_features;


                if (~compute_only_features)
                    [h, angle_bin] = histc(part_features(5), linspace(0, pi/2, 11));
                    [h, overlap_bin] = histc(part_features(1), linspace(0, 1, 11));
                    [h, dilated_overlap_bin] = histc(part_features(6), linspace(0, 1, 11));
                    [h, bearing1_bin] = histc(part_features(3), linspace(0, pi/2, 11));
                    [h, bearing2_bin] = histc(part_features(4), linspace(0, pi/2, 11));
                    [h, appearance_bin] = histc(part_features(9), linspace(0, 1, 11));
                    [h, boundary_bin] = histc(part_features(14), linspace(0, 1, 11));

                    angle_bin = max(1,min(10, angle_bin));
                    overlap_bin = max(1,min(10, overlap_bin));
                    dilated_overlap_bin = max(1,min(10, dilated_overlap_bin));
                    bearing1_bin = max(1,min(10, bearing1_bin));
                    bearing2_bin = max(1,min(10, bearing2_bin));
                    appearance_bin = max(1,min(10, appearance_bin));
                    boundary_bin = max(1,min(10, boundary_bin));

%                     % Method 1 of affinity computation
%                     overlap_affinity = 0;
%                     if ((overlap_bin < 3 && dilated_overlap_bin < 6) || ...
%                         (overlap_bin > 9 && dilated_overlap_bin > 9))
%                         overlap_affinity = 1;
%                     end
%                     
%                     part_affinity(i,j) = overlap_affinity * ...
%                         part_classifier_params.part_appearance_hist(appearance_bin); %part_classifier_params.bearing_hist(bearing1_bin, bearing2_bin) * part_features(8);
%                     part_affinity(j,i) = part_affinity(i,j);
                    
                    % Method 2 of affinity computation
                    same_part_affinity = part_classifier_params.part_appearance_same_part(appearance_bin) * ...
                        part_classifier_params.overlap_boundary_same_part(overlap_bin, boundary_bin) / ...
                        part_classifier_params.max_prob_same_part;
                    
                    diff_part_affinity = part_classifier_params.part_appearance_diff_part(appearance_bin) * ...
                        part_classifier_params.overlap_boundary_diff_part(overlap_bin, boundary_bin) / ...
                        part_classifier_params.max_prob_diff_part;
%                     diff_part_affinity = part_classifier_params.overlap_boundary_diff_part(overlap_bin, boundary_bin);
                    
                    feature_ind = [1, 9, 14];
                    affinity_features = [part_features(feature_ind); 1];
                    prob_duplicate = LogisticPredict(affinity_features', part_classifier_params.logistic_params_duplicate, true);
                    
                    feature_ind = [9, 20];
                    affinity_features = [part_features(feature_ind); 1];
                    affinity_given_non_duplicate1 = LogisticPredict(affinity_features', part_classifier_params.logistic_params1);
                    affinity_given_non_duplicate2 = LogisticPredict(affinity_features', part_classifier_params.logistic_params2);
                    is_end_to_end_junction = part_features(22) == 1;
                    is_t_junction = part_features(22) == 2;
                    is_cross_junction = part_features(22) == 3;
                    is_other_junction = part_features(22) == 4;
                    is_end_to_end_or_other_junction = ~is_t_junction & ~is_cross_junction;
                    affinity_given_non_duplicate = is_t_junction * affinity_given_non_duplicate1 + ...
                        is_cross_junction * 0 + ...
                        is_other_junction * 0 + ...
                        is_end_to_end_junction * affinity_given_non_duplicate2;
                    
                    cur_affinity = prob_duplicate + (1-prob_duplicate)*affinity_given_non_duplicate;
%                     cur_affinity = prob_duplicate*same_part_affinity + (1-prob_duplicate)*diff_part_affinity;
                    
                    part_affinity(i,j) = cur_affinity;
                    part_affinity(j,i) = cur_affinity;
                end

                part_adjacency(i,j) = (part_features(7) > 0);
                part_adjacency(j,i) = (part_features(7) > 0);
            end
        end
    end
    
% %     % Instead of the original affinity, make it the average neighbourhood
% %     % affinity
% %     new_part_affinity = zeros(num_parts, num_parts);
% %     for i = 1:num_parts
% %         for j = 1:i-1
% %             neigh_i = part_adjacency(i, :) > 0;
% %             neigh_i(j) = false;
% %             quality_i = part_affinity(i, neigh_i);
% %             neigh_j = part_adjacency(j, :) > 0;
% %             neigh_j(i) = false;
% %             quality_j = part_affinity(j, neigh_j);
% %             
% %             neighbour_affinity = ((sum(sum(part_affinity(neigh_i, neigh_j) .* ...
% %                 (quality_i' * quality_j) .* part_adjacency(neigh_i, neigh_j)))) + ...
% %                 part_affinity(i,j)) / ...
% %                 (sum(sum((quality_i' * quality_j) .* part_adjacency(neigh_i, neigh_j))) + 1);
% %             
% %             new_part_affinity(i,j) = neighbour_affinity;
% %             new_part_affinity(j,i) = neighbour_affinity;
% %         end
% %     end
% %     
% %     part_affinity = new_part_affinity;