function [junction_point, gap_points] = PartPairIntersection(part1_feature, part2_feature)

    [M,N] = size(part1_feature.boundary);
    half_gap_width = 5;
    
    [part1_side1, part1_side2] = PartSides(part1_feature);
    [part2_side1, part2_side2] = PartSides(part2_feature);
    major_axis1 = PartMajorAxis(part1_feature);
    major_axis2 = PartMajorAxis(part2_feature);
    middle1 = (major_axis1(1:2) + major_axis1(3:4)) / 2;
    middle2 = (major_axis2(1:2) + major_axis2(3:4)) / 2;
    
    [axes_distance_closest, axes_distance_inter, is_projected_in, junc_point] = LineSegmentDistance(major_axis1, major_axis2);
    
    dist_to_middle1 = norm(middle1 - junc_point) / part1_feature.a;
    dist_to_middle2 = norm(middle2 - junc_point) / part2_feature.a;
    
    % Check which part forms a T-junction with the other part
    if (dist_to_middle1 < dist_to_middle2)
        % Part 1 is the upper part of the T junction
        % Intersect part 2 with both sides of part 1 and see which
        % intersection is closer to part 2
        [axes_distance_closest, axes_distance_inter1, is_projected_in, junc_point1] = LineSegmentDistance(major_axis2, part1_side1);
        [axes_distance_closest, axes_distance_inter2, is_projected_in, junc_point2] = LineSegmentDistance(major_axis2, part1_side2);
        if (axes_distance_inter1 < axes_distance_inter2)
            junction_point = junc_point1;
        else
            junction_point = junc_point2;
        end
        
        % Compute the closest boundary point of part 1 to the intersection
        [x,y] = ind2sub([M,N], part1_feature.ordered_boundary);
        num_boundary_pts = numel(part1_feature.ordered_boundary);
        [dist_to_line, junction_point] = PointToLineDist_MEX(major_axis2, [y,x]');
        [min_dist, min_ind] = min(dist_to_line);
        junction_point = junction_point(:, min_ind);
        half_gap_width = round(part2_feature.b);
    else
        % Part 2 is the upper part of the T junction
        % Intersect part 1 with both sides of part 2 and see which
        % intersection is closer to part 1
        [axes_distance_closest, axes_distance_inter1, is_projected_in, junc_point1] = LineSegmentDistance(major_axis1, part2_side1);
        [axes_distance_closest, axes_distance_inter2, is_projected_in, junc_point2] = LineSegmentDistance(major_axis1, part2_side2);
        if (axes_distance_inter1 < axes_distance_inter2)
            junction_point = junc_point1;
        else
            junction_point = junc_point2;
        end
        
        % Compute the closest boundary point of part 1 to the intersection
        [x,y] = ind2sub([M,N], part2_feature.ordered_boundary);
        num_boundary_pts = numel(part2_feature.ordered_boundary);
        [dist_to_line, junction_point] = PointToLineDist_MEX(major_axis1, [y,x]');
        [min_dist, min_ind] = min(dist_to_line);
        junction_point = junction_point(:, min_ind);
        half_gap_width = round(part1_feature.b);
    end

    sq_dist_to_boundary = (junction_point(1) - y).^2 + ...
        (junction_point(2) - x).^2;
    [min_dist, min_ind] = min(sq_dist_to_boundary);
    gap_ind = (min_ind-half_gap_width):(min_ind+half_gap_width);
    gap_ind = mod(gap_ind-1, num_boundary_pts)+1;
    gap_points = [y(gap_ind), x(gap_ind)];
    