#include <stdio.h>
#include <math.h>
#include "mex.h"

double max(double num1, double num2) {
    return num1 > num2 ? num1: num2;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,
		const mxArray *prhs[])
{
	double *line, *points;
	double *distance, *junction_pt;
    mxLogical *inside_ind;
    double *neigh_point;
    
	int num_points = mxGetN(prhs[1]);
	
	line = mxGetPr(prhs[0]);
	points = mxGetPr(prhs[1]);

    plhs[0] = mxCreateDoubleMatrix(num_points, 1, mxREAL);
    if (nlhs > 1)
    {
        plhs[1] = mxCreateDoubleMatrix(2, num_points, mxREAL);
        plhs[2] = mxCreateLogicalMatrix(num_points, 1);
        plhs[3] = mxCreateDoubleScalar(0);
    }
    
    distance = mxGetPr(plhs[0]);
    if (nlhs > 1)
    {
        junction_pt = mxGetPr(plhs[1]);
        inside_ind = mxGetLogicals(plhs[2]);
        neigh_point = mxGetPr(plhs[3]);
    }
    
    for (int pt = 0; pt < num_points; pt++) {
        double pt_x = *(points + 2*pt + 0);
        double pt_y = *(points + 2*pt + 1);
        double pt1_x = line[0];
        double pt1_y = line[1];
        double pt2_x = line[2];
        double pt2_y = line[3];
        
        double line_length_sq = ((pt1_x - pt2_x) * (pt1_x - pt2_x) +
            (pt1_y - pt2_y) * (pt1_y - pt2_y));
        double line_proj = ((pt2_x - pt1_x) * (pt_x - pt1_x) + 
            (pt2_y - pt1_y) * (pt_y - pt1_y));

        if (nlhs > 1)
        {
            inside_ind[pt] = false;
        }
        
        if (line_proj < 0) {
            double dist_to_pt1 = sqrt((pt_x - pt1_x) * (pt_x - pt1_x) + 
                (pt_y - pt1_y) * (pt_y - pt1_y));
            distance[pt] = dist_to_pt1;
            
            if (nlhs > 1)
            {
                *(junction_pt + 2*pt + 0) = (pt_x + pt1_x) / 2;
                *(junction_pt + 2*pt + 1) = (pt_y + pt1_y) / 2;
                *neigh_point = 1;
            }
        }
        else if (line_proj > line_length_sq) {
            double dist_to_pt2 = sqrt((pt_x - pt2_x) * (pt_x - pt2_x) + 
                (pt_y - pt2_y) * (pt_y - pt2_y));
            distance[pt] = dist_to_pt2;

            if (nlhs > 1)
            {
                *(junction_pt + 2*pt + 0) = (pt_x + pt2_x) / 2;
                *(junction_pt + 2*pt + 1) = (pt_y + pt2_y) / 2;
                *neigh_point = 2;
            }
        }
        else {
            double dist_to_pt1_sq = ((pt_x - pt1_x) * (pt_x - pt1_x) + 
                (pt_y - pt1_y) * (pt_y - pt1_y));
            double dist_to_line = sqrt(max(dist_to_pt1_sq - line_proj*line_proj/line_length_sq, 0));
            distance[pt] = dist_to_line;
            
            if (nlhs > 1)
            {
                double frac_from_1 = line_proj/line_length_sq;
                *(junction_pt + 2*pt + 0) = ((1-frac_from_1)*pt1_x + frac_from_1 * pt2_x);
                *(junction_pt + 2*pt + 1) = ((1-frac_from_1)*pt1_y + frac_from_1 * pt2_y);
                inside_ind[pt] = true;
            }
        }
    }
}
