function [dist_to_line, junction_point, inside_ind, point_neigh] = PointToLineDist_MEX(line_seg, point)

% PointToLineDist_MEX - Calculate the distance between points and a line
% segment
%
% dist_to_line = PointToLineDist_MEX(line_seg, point)
% Given the two endpoints of a line segments and a collection of points,
% calculate the distance of each of the points to the line segment
%
% Arguments:
%   line_seg - 4x1 vector representing the line segment, where
%   line_seg(1:2) are the x,y coordinates of the 1st point and
%   line_seg(3:4) are the coordinates of the 2nd point
%   point - 2xN matrix of points. point(:,i) is the i-th point
%
% Returns:
%   dist_to_line - Nx1 vector of distances of points to the line segment
%   junction_point - 2xN matrix of junction points. If the projection falls
%   onto the line, the junction point is that point on the line segment. If
%   not, then the junction point is the middle point between the point end
%   one of the line's endpoints.
%   inside_ind - Nx1 logical array that indicates whether each point is
%   projected inside the segment or not.
%   point_neigh - Nx1 array of closest line endpoint indices. 0 means the
%   the point projected inside the line. 1 - closest to the first endpoint.
%   2- closest to the second endpoint.

