% Compute the boundary of the bigest binary region in BW

function boundary = RegionBoundary(BW)

    [M,N] = size(BW);
    components = bwlabel(BW);
    components_stats = regionprops(components, 'Area');
    components_area = [components_stats.Area];
    [m, max_area_ind] = max(components_area);
    BW = components == max_area_ind;
    B = bwboundaries(BW,8,'noholes');
    boundary = sub2ind([M,N], B{1}(:,1), B{1}(:,2));

    num_pixels = numel(boundary);
    [rows, cols] = ind2sub([M,N], boundary);
    adjmat = zeros(num_pixels, num_pixels);

    for i = 1:num_pixels
        for j = 1:i-1
            if (abs(rows(i) - rows(j)) <= 1 && abs(cols(i) - cols(j)) <= 1)
                adjmat(i,j) = 1;
                adjmat(j,i) = 1;
            end
        end
    end

    bad_pixels = false(num_pixels, 1);
    for i = 1:num_pixels
        if (bad_pixels(i))
            continue;
        end

        neighs = find(adjmat(:,i) & ~bad_pixels);
        if (numel(neighs) > 2)
            % Find the furthest neighbour
            [m, far_n] = max(min(num_pixels - abs(neighs - i), abs(neighs - i)));
            far_n = neighs(far_n);

            if (abs(far_n - i) < num_pixels/2)
                bad_pixels((min(far_n, i)+1):(max(far_n, i)-1)) = true;
            else
                bad_pixels((max(far_n, i)+1):end) = true;
                bad_pixels(1:(min(far_n, i)-1)) = true;
            end
        end
    end

    boundary = boundary(~bad_pixels);
