% Compute the edge distribution as a function of arc length (location on
% the perimeter)
%
function [symmetry_feature, supporting_stats] = RegionSymmetryFeature4(boundary, shape_pixels, pb, dilated_pb, display_on)

    [y,x] = ind2sub(size(pb), boundary);
%     normal = BoundaryNormal([x,y]);
    BW = zeros(size(pb));
    BW(shape_pixels) = 1;
    stats = regionprops(BW, 'Orientation', 'MajorAxisLength', 'MinorAxisLength', 'Centroid');
    a = stats(1).MajorAxisLength/2;
    b = stats(1).MinorAxisLength/2;
    x0 = stats(1).Centroid(1);
    y0 = stats(1).Centroid(2);
    ellipse_angle = -pi*stats(1).Orientation/180;
    
    if (display_on)
        clf;
        B = zeros(size(pb));
        B(boundary) = dilated_pb(boundary);
    
        subplot(1,2,1);
        imagesc(pb);
        
        subplot(1,2,2);
        imagesc(BW);
        
        t = 0:0.01:2*pi;
        el_x = a*cos(t);
        el_y = b*sin(t);
        el_x_new = el_x*cos(ellipse_angle) - el_y*sin(ellipse_angle);
        el_y_new = el_x*sin(ellipse_angle) + el_y*cos(ellipse_angle);
        el_x = el_x_new + x0;
        el_y = el_y_new + y0;
        
        hold on;
        plot(el_x,el_y);
        quiver(x0, y0, 10*cos(ellipse_angle), 10*sin(ellipse_angle), 'g', 'LineWidth', 4);
    end
    
    major_vector = [cos(ellipse_angle); sin(ellipse_angle)];
    minor_vector = [cos(ellipse_angle + pi/2); sin(ellipse_angle + pi/2)];
    %         normal_angle = mod(angle(complex(normal(:,1), normal(:,2))) + ellipse_angle, 2*pi);
    boundary_strength = double(dilated_pb(boundary) > 0.01);

    boundary_major_projection = ((x - x0)*major_vector(1) + ...
        (y - y0)*major_vector(2)) / a;
    boundary_minor_projection = ((x - x0)*minor_vector(1) + ...
        (y - y0)*minor_vector(2)) / b;

    anisotropy = b/a;
    ellipse_perimeter = pi * (3*(a+b) - sqrt((a+3*b)*(3*a+b)));
    major_minor_hist = hist_twoD(boundary_major_projection, boundary_minor_projection, -1.5:0.3:1.5, -1.5:0.3:1.5, boundary_strength)';
%     major_minor_hist_count = hist_twoD(boundary_major_projection, boundary_minor_projection, -1.5:0.3:1.5, -1.5:0.3:1.5);
    major_minor_hist = major_minor_hist ./ (numel(boundary) + eps);
    major_minor_hist = major_minor_hist(1:end-1,1:end-1);
%     major_minor_dct = dct2(major_minor_hist);
    minor_hist = mean(major_minor_hist, 2);
    major_hist = mean(major_minor_hist, 1);
%     major_minor_hist = imfilter(major_minor_hist, fspecial('gaussian',[3,3],1), 'replicate');
%     major_minor_hist_sym = sqrt(major_minor_hist .* major_minor_hist(end:-1:1,:));
%     major_minor_hist_sym = sqrt(major_minor_hist_sym .* major_minor_hist_sym(:,end:-1:1));
    
    symmetry_feature = [major_minor_hist(:); major_hist(:); minor_hist(:); anisotropy;];
    supporting_stats.boundary_strength = boundary_strength;
    supporting_stats.ellipse.a = a;
    supporting_stats.ellipse.b = b;
    supporting_stats.ellipse.x0 = x0;
    supporting_stats.ellipse.y0 = y0;
    supporting_stats.ellipse.angle = ellipse_angle;
    supporting_stats.boundary = boundary;
    
    if (display_on)
        %             B = zeros(size(pb));
        %             B(circshift(boundary, -min_dist_ind)) = 1:numel(boundary);
        %             imagesc(B);
        %             hold on;
        %
%         figure(2);
%         imagesc(major_minor_dct);
%         colorbar;
%         figure(1);
        pause;
    end
