% SuperpixelAffinityGetTrainedAffinity - Uses a classifier on pairwise
% features to come up with affinities for all adjacent superpixels
%
% affinity = SuperpixelAffinityGetTrainedAffinity(classifier_params,
% adjmat, pairwise_features)
% Given the parameters of an already trained classifier, the superpixel
% adjacency matrix and the pairwise features between adjacent superpixels,
% calculate the affinity between adjacent superpixels.
%
% Arguments:
%   classifier_params - A structure storing the parameters of the
%   classifier (either SVM or logistic)
%   adjmat - num_sups x num_sups adjacency matrix between superpixels
%   pairwise_features - num_sups x num_sups x num_features array of
%   pairwise features. pairwise_features(:,:,i) is the i-th pairwise
%   feature
%
function affinity = SuperpixelAffinityGetTrainedAffinity(classifier_params, adjmat, pairwise_features)

    [sup1, sup2] = find(tril(adjmat, -1));
    num_edges = numel(sup1);
    affinity = zeros(size(adjmat));
    
    for i=1:num_edges
        cur_features = squeeze(pairwise_features(sup1(i), sup2(i), :))';
        
        cur_affinity = SuperpixelRegionAffinityGetTrainedAffinity(cur_features, classifier_params);
        
        affinity(sup1(i), sup2(i)) = cur_affinity;
        affinity(sup2(i), sup1(i)) = cur_affinity;
    end