% SuperpixelAffinityTextureHistogram - Compute pairwise superpixel
% affinities based on texture histograms
%
% texture_hist_affinities = SuperpixelAffinityTextureHistogram(img, seg_image, adjmat)
% Extracts texture histograms for every superpixel. Compares the histograms
% of adjacent superpixels and returns the affinities.
%
% Arguments:
%   img - MxNx3 input image
%   seg_image - MxN superpixel segmentation storing superpixel indeces in
%   each pixel (0 if on the boundary or background).
%   adjmat - num_sups x num_sups matrix where num_sups = max(seg_image(:))
%   is the number of superpixels. 1 indicates that the corresponding
%   superpixels are neighbouring.
%
% Returns:
%   texture_hist_affinities - num_sups x num_sups matrix of superpixel
%   affinities. Except for the diagonal, only entries corresponding to
%   adjacent superpixels are filled.
%
function texture_hist_affinities = SuperpixelAffinityTextureHistogram(img, seg_image, adjmat)

    texture_hist_affinities = eye(size(adjmat));
    
    [sup1, sup2] = find(tril(adjmat, -1));
    num_sup_edges = numel(sup1);
    
    no = 6;
    ss = 1;
    ns = 2;
    sc = sqrt(2);
    el = 2;
    k = 32;
    fname = sprintf( ...
        'unitex_%.2g_%.2g_%.2g_%.2g_%.2g_%d.mat',no,ss,ns,sc,el,k);
    load(fname,'tex','fb','tsim'); % defines fb,tex,tsim
    tmap = assignTextons(fbRun(fb,rgb2gray(img)),tex);
    
    K = max(seg_image(:));
    tex_range = (1:(k+1)) - 0.5;
    tex_hist = zeros(K, k);
    
    for i = 1:K
        tex_hist(i, :) = RegionTextureHistogram(seg_image == i, tmap, tex_range);
    end

    for sup_edge = 1:num_sup_edges
        cur_affinity = AffinityTextureHistogram(tex_hist(sup1(sup_edge), :), tex_hist(sup2(sup_edge), :));
        
        texture_hist_affinities(sup1(sup_edge), sup2(sup_edge)) = cur_affinity;
        texture_hist_affinities(sup2(sup_edge), sup1(sup_edge)) = cur_affinity;
    end

    