% SuperpixelBoundaries - Compute the boundary pixels of every superpixel
%
% sup_boundaries = SuperpixelBoundaries(seg_image, boundary)
% For every superpixel in seg_image, find the boundaries in boundary which
% are the boundaries of that superpixel. Return a list of linear boundary
% pixel indices for each superpixel.
%
% Arguments:
%   seg_image - MxN superpixel segmentation
%  
% Returns:
%   sup_boundaries - num_sups x 1 cell array of superpixel boundaries. Each
%   entry contains a list of linear pixel indices of the boundary pixels
%   for the given superpixel.
%
function sup_boundaries = SuperpixelBoundaries(seg_image)

    [M,N] = size(seg_image);
    num_sups = max(seg_image(:));
    sup_boundaries = cell(num_sups, 1);
    
    for sup = 1:num_sups
        BW = seg_image == sup;
        sup_boundaries{sup} = RegionBoundary(BW);
%         b = zeros(M,N);
%         b(sup_boundaries{sup}) = (1:numel(sup_boundaries{sup})) + 10;
%         imagesc(b);
%         pause;
    end

 