% ---------
% hist_twoD
% ---------
%
% Calculates a XxY matrix of counts.
%
% function [z, Xgrid, Ygrid] = hist_twoD(X,Y,xspace,yspace)
%
% Parameters
% X:       a matrix containing X coordinate values
% Y:       a matrix containing Y coordinate values
% xspace:  histogram edges in the X coordinate
% yspace:  histogram edges in the Y coordinate
%
% Outputs
% z:       a matrix containing two-D histogram counts
% Xgrid:   a matrix containing X edge positions for the histogram
% Ygrid:   a matrix containing Y edge positions for the histogram

function [z, Xgrid, Ygrid] = hist_twoD(X,Y,xspace,yspace, weights)

%%% Change the matricies into vectors for convenience
X = X(1:end);
Y = Y(1:end);

%%% Default x and y ranges if xspace and yspace are not given
if nargin < 4
  minX = min(X);
  maxX = max(X);
  xspace = linspace(minX,maxX,50);
  xsp = xspace(2) - xspace(1);
  minY = min(Y);
  maxY = max(Y);
  yspace = linspace(minY,maxY,50);
  ysp  = yspace(2) - yspace(1);
else
  maxX = max(xspace);
  minX = min(xspace);
  xsp = xspace(2) - xspace(1);
  maxY = max(yspace);
  minY = min(yspace);
  ysp  = yspace(2) - yspace(1);
end

if (nargin < 5)
    weights = ones(size(X));
end

%%% Find the index of all points
xind = floor((X-minX)/xsp)+1;
yind = floor((Y-minY)/ysp)+1;

z = zeros(length(xspace),length(yspace));

for i = 1:numel(X)
    %%% If the point is within the range,
    if (xind(i)>0) & (xind(i)<(length(xspace)+1)) & ...
            (yind(i)>0)&(yind(i)<(length(yspace)+1))
      %%% increment the appropriate count
      try
          z(xind(i),yind(i)) = z(xind(i),yind(i))+weights(i);
      catch
          rowidx = min(xind(i),size(z,1));
          colidx = min(yind(i),size(z,2));
 
          z(rowidx,colidx) = z(rowidx,colidx) + weights(i);
      end
    end
end
%%% Generate edge position matricies
[Xgrid,Ygrid]=meshgrid(xspace,yspace);